#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const inquirer_1 = __importDefault(require("inquirer"));
const handlebars_1 = __importDefault(require("handlebars"));
const del_1 = require("./del"); // 导入路径保持小写以符合规范
const commander_1 = require("commander");
const simple_git_1 = require("simple-git");
const promise_1 = __importDefault(require("git-clone/promise"));
const config_1 = require("./config");
const program = new commander_1.Command();
const { version } = require('../package.json');
/**
 * 配置命令行程序
 * @param program Commander实例
 * @param version 当前版本号
 */
program
    .name('npm-init-cli')
    .description('CLI for initializing game npm packages') // 描述改为英文保持统一
    .version(version, '-v, --version', 'output current version');
program
    .command('init <name> [gitBranch]')
    .description('Initialize a new game project')
    .action((name, gitBranch) => __awaiter(void 0, void 0, void 0, function* () {
    //git分支
    gitBranch = gitBranch || 'main';
    // 验证项目名称格式
    if (!/^[a-z_]+$/.test(name)) {
        throw new Error('Project name can only contain lowercase letters and hyphens');
    }
    const { branch } = yield inquirer_1.default.prompt(config_1.initPrompts);
    const clonePath = path_1.default.resolve(process.cwd(), name); // 使用resolve防止路径遍历
    console.log(`Project path:${clonePath}`);
    try {
        yield (0, del_1.removeDirOrFile)(clonePath);
    }
    catch (error) {
        throw new Error(`Failed to clean target directory: ${error.message}`);
    }
    try {
        // 克隆仓库并显示进度
        console.log(`Cloning template from ${config_1.gitPath}...`);
        yield (0, promise_1.default)(config_1.gitPath, clonePath, {
            checkout: branch
            //shallow: true // 浅克隆优化性能
        });
    }
    catch (error) {
        throw new Error(`Git clone failed: ${error.message}`);
    }
    // 删除.git目录
    const gitDir = path_1.default.join(clonePath, '.git');
    try {
        yield (0, del_1.removeDirOrFile)(gitDir);
    }
    catch (error) {
        throw new Error(`Failed to remove .git directory: ${error.message}`);
    }
    const packagePath = path_1.default.join(clonePath, 'package.json');
    if (fs_1.default.existsSync(packagePath)) {
        /**
         * 处理package.json模板
         * @param filePath 模板文件路径
         * @param projectName 项目名称
         * @returns 处理后的文件内容
         */
        const processTemplate = (filePath, projectName) => __awaiter(void 0, void 0, void 0, function* () {
            try {
                const content = yield fs_1.default.promises.readFile(filePath, 'utf-8');
                const template = handlebars_1.default.compile(content);
                return template({ name: projectName });
            }
            catch (error) {
                throw new Error(`Template processing failed: ${error.message}`);
            }
        });
        const result = yield processTemplate(packagePath, name);
        // 校验生成的package.json格式
        try {
            JSON.parse(result);
        }
        catch (e) {
            throw new Error(`Generated package.json is invalid: ${e.message}`);
        }
        yield fs_1.default.promises.writeFile(packagePath, result);
        console.log('package.json updated successfully');
        /**
         * 初始化Git仓库并配置远程地址
         * @param clonePath 项目克隆路径
         * @param name 项目名称
         */
        const git = (0, simple_git_1.simpleGit)(clonePath);
        yield git.init();
        const repoUrl = `http://ser.yinengyun.com:10082/npm/${name}.git`;
        try {
            // 配置远程仓库并重命名默认分支
            yield git.remote(['add', 'origin', repoUrl]);
            yield git.branch(['-M', gitBranch]);
            yield git.pull(repoUrl, gitBranch);
        }
        catch (error) {
            throw new Error(`Failed to configure git remote: ${error.message}`);
        }
    }
    else {
        throw new Error('no package.json');
    }
}));
program.parse();
//# sourceMappingURL=index.js.map