#!/usr/bin/env node
import fs from 'fs';
import path from 'path';
import inquirer from 'inquirer';
import handlebars from 'handlebars';
import { removeDirOrFile } from "./del"; // 导入路径保持小写以符合规范
import { Command } from 'commander';
import { simpleGit } from 'simple-git';
import gitclone from 'git-clone/promise';
import { initPrompts, gitPath } from './config';

const program = new Command();

const { version } = require('../package.json');

/**
 * 配置命令行程序
 * @param program Commander实例
 * @param version 当前版本号
 */
program
    .name('npm-init-cli')
    .description('CLI for initializing game npm packages') // 描述改为英文保持统一
    .version(version, '-v, --version', 'output current version');

program
    .command('init <name> [gitBranch]')
    .description('Initialize a new game project')
    .action(async (name: string, gitBranch?: string) => {

        //git分支
        gitBranch = gitBranch || 'main';

        // 验证项目名称格式
        if (!/^[a-z_]+$/.test(name)) {
            throw new Error('Project name can only contain lowercase letters and hyphens');
        }

        const { branch } = await inquirer.prompt<{ branch: string }>(initPrompts as any);

        const clonePath = path.resolve(process.cwd(), name); // 使用resolve防止路径遍历

        console.log(`Project path:${clonePath}`)

        try {
            await removeDirOrFile(clonePath);
        } catch (error) {
            throw new Error(`Failed to clean target directory: ${(error as Error).message}`);
        }
        try {
            // 克隆仓库并显示进度
            console.log(`Cloning template from ${gitPath}...`);
            await gitclone(gitPath, clonePath, {
                checkout: branch
                //shallow: true // 浅克隆优化性能
            });
        } catch (error) {
            throw new Error(`Git clone failed: ${(error as Error).message}`);
        }

        // 删除.git目录
        const gitDir = path.join(clonePath, '.git');
        try {
            await removeDirOrFile(gitDir);
        } catch (error) {
            throw new Error(`Failed to remove .git directory: ${(error as Error).message}`);
        }
        const packagePath = path.join(clonePath, 'package.json');
        if (fs.existsSync(packagePath)) {
            /**
             * 处理package.json模板
             * @param filePath 模板文件路径
             * @param projectName 项目名称
             * @returns 处理后的文件内容
             */
            const processTemplate = async (filePath: string, projectName: string) => {
                try {
                    const content = await fs.promises.readFile(filePath, 'utf-8');
                    const template = handlebars.compile(content);
                    return template({ name: projectName });
                } catch (error) {
                    throw new Error(`Template processing failed: ${(error as Error).message}`);
                }
            };

            const result = await processTemplate(packagePath, name);

            // 校验生成的package.json格式
            try {
                JSON.parse(result);
            } catch (e) {
                throw new Error(`Generated package.json is invalid: ${(e as Error).message}`);
            }

            await fs.promises.writeFile(packagePath, result);
            console.log('package.json updated successfully');

            /**
             * 初始化Git仓库并配置远程地址
             * @param clonePath 项目克隆路径
             * @param name 项目名称
             */
            const git = simpleGit(clonePath);
            await git.init();

            const repoUrl = `http://ser.yinengyun.com:10082/npm/${name}.git`;

            try {
                // 配置远程仓库并重命名默认分支
                await git.remote(['add', 'origin', repoUrl]);
                await git.branch(['-M', gitBranch]);
                await git.pull(repoUrl, gitBranch);
            } catch (error) {
                throw new Error(`Failed to configure git remote: ${(error as Error).message}`);
            }
        } else {
            throw new Error('no package.json');
        }
    })

program.parse();
